# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: token.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0btoken.proto\"J\n\x05Token\x12\x0f\n\x07user_id\x18\x01 \x01(\r\x12\x12\n\nproject_id\x18\x02 \x01(\r\x12\x0f\n\x07methods\x18\x03 \x01(\t\x12\x0b\n\x03\x65xp\x18\x04 \x01(\x01\x62\x06proto3')



_TOKEN = DESCRIPTOR.message_types_by_name['Token']
Token = _reflection.GeneratedProtocolMessageType('Token', (_message.Message,), {
  'DESCRIPTOR' : _TOKEN,
  '__module__' : 'token_pb2'
  # @@protoc_insertion_point(class_scope:Token)
  })
_sym_db.RegisterMessage(Token)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _TOKEN._serialized_start=15
  _TOKEN._serialized_end=89
# @@protoc_insertion_point(module_scope)
